import channel
import playlist
import video


def example_channel():
    channel1 = channel.Channel("https://www.youtube.com/@PewDiePie")

    print(channel1.name)  # PewDiePie
    print(channel1.description)  # I make videos.
    print(channel1.channel_url)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw
    print(channel1.profile_photo_url)  # https://yt3.googleusercontent.com/5oUY3tashyxfqsjO5SGhjT4dus8FkN9CsAHwXWISFrdPYii1FudD4ICtLfuCw6-THJsJbgoY=s900-c-k-c0x00ffffff-no-rj
    print(channel1.channel_id)  # UC-lHJZR3Gqxm24_Vd_AJ5Yw

    # The following two variables may produce different results depending on your geolocation.
    print(channel1.subscribers_count)
    print(channel1.video_count)

    print(channel1.videos_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/videos
    print(channel1.shorts_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/shorts
    print(channel1.community_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/community
    print(channel1.featured_channels_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/channels
    print(channel1.playlists_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/playlists
    print(channel1.about_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/about


def example_video():
    video1 = video.Video("https://youtu.be/XqZsoesa55w")

    print(video1.title)  #  Baby Shark Dance | #babyshark Most Viewed Video | Animal Songs | PINKFONG Songs for Children
    print(video1.description)  # a loooong description :)
    print(video1.author)  # Baby Shark - Pinkfong Kids’ Songs & Stories
    print(video1.video_url)  # https://www.youtube.com/watch?v=XqZsoesa55w

    print(video1.views)  # 14941899516
    print(video1.thumbnail_url)  # https://i.ytimg.com/vi/XqZsoesa55w/maxresdefault.jpg
    print(video1.date_published)  # 2016-06-17 16:00:30-07:00
    print(video1.legth_seconds)  # 136
    print(video1.isLiveContent)  # False
    print(video1.is_private)  # False
    print(video1.genre)  # Education


def example_shorts_video():
    video2 = video.Video("https://www.youtube.com/shorts/JfbnpYLe3Ms")

    print(video2.title)
    print(video2.description)
    print(video2.author)
    print(video2.video_url)

    print(video2.views)
    print(video2.thumbnail_url)
    print(video2.date_published)
    print(video2.legth_seconds)
    print(video2.isLiveContent)
    print(video2.is_private)
    print(video2.genre)


def example_playlist():
    playlist1 = playlist.Playlist("https://www.youtube.com/watch?v=K4DyBUG242c&list=PLRBp0Fe2Gpgm_u2w2a2isHw29SugZ34cD&ab_channel=NoCopyrightSounds")
    print(playlist1.title)  # NCS : The Top 100 Biggest Songs 📈
    print(playlist1.author)  # NoCopyrightSounds
    print(playlist1.total_videos)  # 100
    print(playlist1.playlist_id)  # PLRBp0Fe2Gpgm_u2w2a2isHw29SugZ34cD
    print(playlist1.playlist_url)  # https://www.youtube.com/playlist?list=PLRBp0Fe2Gpgm_u2w2a2isHw29SugZ34cD
    print(playlist1.video_urls)  # ['https://youtube.com/watch/?v=K4DyBUG242c', 'https://youtube.com/watch/?v=3nQNiWdeH2Q', 'https://youtube.com/watch/?v=J2X5mJ3HDYE', ... ]


example_channel()
example_video()
example_shorts_video()
example_playlist()



import urllib.parse
import urllib.request

from bs4 import BeautifulSoup

from channel import Channel
from funcs import get_initial_data
from video import Video


class Playlist:
    def __init__(self, url="") -> None:
        response = urllib.request.urlopen(url).read().decode("utf8")
        soup = BeautifulSoup(response, "html.parser")
        initial_data = get_initial_data(str(soup.find_all("script")[45]))
        self.playlist_data = initial_data["contents"]["twoColumnWatchNextResults"]["playlist"]["playlist"]


        self.title: str = self.playlist_data["title"]
        self.author: str = self.playlist_data["ownerName"]["simpleText"]
        self.playlist_id: str = self.playlist_data["playlistId"]
        self.playlist_url: str = f"https://www.youtube.com/playlist?list={self.playlist_id}"
        self.total_videos: int = int(self.playlist_data["totalVideos"])
        self.is_infinite: bool = self.playlist_data["isInfinite"]

        self.video_urls: list[str] = [self.get_video_url(i) for i in range(self.total_videos)]

    def get_video_url(self, video_id: int) -> Video:
        video = self.playlist_data["contents"][video_id]["playlistPanelVideoRenderer"]["navigationEndpoint"]["watchEndpoint"]["videoId"]
        return f"https://youtube.com/watch/?v={video}"



import json
import re

from graphviz import Digraph


def get_initial_data(script_content: str) -> str | Exception:
    pattern = re.compile(r"var ytInitialData = ({.*?});", re.DOTALL)
    match = pattern.search(script_content)

    if match:
        json_str = match.group(1)
        return json.loads(json_str)
    else:
        raise Exception("ytInitialData not found.")


def get_initial_player_response(script_content: str) -> str | Exception:
    pattern = re.compile(r"var ytInitialPlayerResponse = ({.*?});", re.DOTALL)
    match = pattern.search(script_content)

    if match:
        json_str = match.group(1)
        return json.loads(json_str)
    else:
        raise Exception("ytInitialPlayerResponse not found.")


def visualize_dict(d, parent_key=None, graph=None):
    if graph is None:
        graph = Digraph()

    for key, value in d.items():
        node_key = f"{parent_key}.{key}" if parent_key else str(key)
        graph.node(node_key, str(key))

        if parent_key:
            graph.edge(parent_key, node_key)

        if isinstance(value, dict):
            visualize_dict(value, node_key, graph)
        else:
            value_node = f"{node_key}.value"
            graph.node(value_node, str(value))
            graph.edge(node_key, value_node)

    return graph
